@@ -65,7 +65,8 @@ our $VERSION = $ZoneMinder::Base::VERSION;
 # ==========================================================================
 
 use ZoneMinder::Logger qw(:all);
-use ZoneMinder::Config qw(:all);
+
+require ZoneMinder::Config;
 
 our $dbh = undef;
 
@@ -87,25 +88,26 @@ sub zmDbConnect {
         $socket = ';host='.$host.';port='.$portOrSocket;
       }
     } else {
-      $socket = ';host='.$Config{ZM_DB_HOST}; 
+      $socket = ';host='.$ZoneMinder::Config::Config{ZM_DB_HOST}; 
     }
 
     my $sslOptions = '';
-    if ( $Config{ZM_DB_SSL_CA_CERT} ) {
-      $sslOptions = join(';','',
+    if ( $ZoneMinder::Config::Config{ZM_DB_SSL_CA_CERT} ) {
+      $sslOptions = join(';', '',
           'mysql_ssl=1',
-          'mysql_ssl_ca_file='.$Config{ZM_DB_SSL_CA_CERT},
-          'mysql_ssl_client_key='.$Config{ZM_DB_SSL_CLIENT_KEY},
-          'mysql_ssl_client_cert='.$Config{ZM_DB_SSL_CLIENT_CERT}
+          'mysql_ssl_ca_file='.$ZoneMinder::Config::Config{ZM_DB_SSL_CA_CERT},
+          'mysql_ssl_client_key='.$ZoneMinder::Config::Config{ZM_DB_SSL_CLIENT_KEY},
+          'mysql_ssl_client_cert='.$ZoneMinder::Config::Config{ZM_DB_SSL_CLIENT_CERT}
           );
     }
 
     eval {
       $dbh = DBI->connect(
-        'DBI:mysql:database='.$Config{ZM_DB_NAME}
+        'DBI:mysql:database='.$ZoneMinder::Config::Config{ZM_DB_NAME}
         .$socket . $sslOptions . ($options?join(';', '', map { $_.'='.$$options{$_} } keys %{$options} ) : '')
-        , $Config{ZM_DB_USER}
-        , $Config{ZM_DB_PASS}
+        , $ZoneMinder::Config::Config{ZM_DB_USER}
+        , $ZoneMinder::Config::Config{ZM_DB_PASS}
+        , { mysql_enable_utf8 => 1, }
         );
     };
     if ( !$dbh or $@ ) {
@@ -124,7 +126,7 @@ sub zmDbConnect {
 } # end sub zmDbConnect
 
 sub zmDbDisconnect {
-  if ( defined( $dbh ) ) {
+  if ( defined($dbh) ) {
     $dbh->disconnect() or Error('Error disconnecting db? ' . $dbh->errstr());
     $dbh = undef;
   }
@@ -145,15 +147,15 @@ sub zmDbGetMonitors {
 
   if ( $function ) {
     if ( $function == DB_MON_CAPT ) {
-      $sql .= " where Function >= 'Monitor'";
+      $sql .= " WHERE `Function` >= 'Monitor'";
     } elsif ( $function == DB_MON_ACTIVE ) {
-      $sql .= " where Function > 'Monitor'";
+      $sql .= " WHERE `Function` > 'Monitor'";
     } elsif ( $function == DB_MON_MOTION ) {
-      $sql .= " where Function = 'Modect' or Function = 'Mocord'";
+      $sql .= " WHERE `Function` = 'Modect' OR `Function` = 'Mocord'";
     } elsif ( $function == DB_MON_RECORD ) {
-      $sql .= " where Function = 'Record' or Function = 'Mocord'";
+      $sql .= " WHERE `Function` = 'Record' OR `Function` = 'Mocord'";
     } elsif ( $function == DB_MON_PASSIVE ) {
-      $sql .= " where Function = 'Nodect'";
+      $sql .= " WHERE `Function` = 'Nodect'";
     }
   }
   my $sth = $dbh->prepare_cached( $sql );
@@ -213,6 +215,7 @@ sub zmDbGetMonitor {
     return undef;
   }
   my $monitor = $sth->fetchrow_hashref();
+  $sth->finish();
   return $monitor;
 }
 
@@ -239,6 +242,7 @@ sub zmDbGetMonitorAndControl {
     return undef;
   }
   my $monitor = $sth->fetchrow_hashref();
+  $sth->finish();
   return $monitor;
 }
 
@@ -263,15 +267,14 @@ sub end_transaction {
 } # end sub end_transaction
 
 # Basic execution of $dbh->do but with some pretty logging of the sql on error.
-# Returns 1 on success, 0 on error
 sub zmDbDo {
 	my $sql = shift;
-	if ( ! $dbh->do($sql, undef, @_) ) {
+  my $rows = $dbh->do($sql, undef, @_);
+	if ( ! defined $rows ) {
 		$sql =~ s/\?/'%s'/;
 		Error(sprintf("Failed $sql :", @_).$dbh->errstr());
-    return 0;
 	}
-  return 1;
+  return $rows;
 }
 
 1;
