@@ -24,7 +24,6 @@
 #include "zm_ffmpeg.h"
 #include "zm_packet.h"
 #include "zm_signal.h"
-#include <sys/time.h>
 
 PacketQueue::PacketQueue():
   video_stream_id(-1),
@@ -33,7 +32,8 @@ PacketQueue::PacketQueue():
   max_stream_id(-1),
   packet_counts(nullptr),
   deleting(false),
-  keep_keyframes(false)
+  keep_keyframes(false),
+  warned_count(0)
 {
 }
 
@@ -87,61 +87,6 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
   {
     std::unique_lock<std::mutex> lck(mutex);
 
-    if (add_packet->packet.stream_index == video_stream_id) {
-      if ((max_video_packet_count > 0) and (packet_counts[video_stream_id] > max_video_packet_count)) {
-        Warning("You have set the max video packets in the queue to %u."
-            " The queue is full. Either Analysis is not keeping up or"
-            " your camera's keyframe interval is larger than this setting."
-            " We are dropping packets.", max_video_packet_count);
-        if (add_packet->keyframe) {
-          // Have a new keyframe, so delete everything
-          while ((*pktQueue.begin() != add_packet) and (packet_counts[video_stream_id] > max_video_packet_count)) {
-            std::shared_ptr <ZMPacket>zm_packet = *pktQueue.begin();
-            ZMLockedPacket *lp = new ZMLockedPacket(zm_packet);
-            if (!lp->trylock()) {
-              Debug(1, "Found locked packet when trying to free up video packets. Can't continue");
-              delete lp;
-              break;
-            }
-            delete lp;
-
-            for (
-                std::list<packetqueue_iterator *>::iterator iterators_it = iterators.begin();
-                iterators_it != iterators.end();
-                ++iterators_it
-                ) {
-              packetqueue_iterator *iterator_it = *iterators_it;
-              // Have to check each iterator and make sure it doesn't point to the packet we are about to delete
-              if ( *(*iterator_it) == zm_packet ) {
-                Debug(1, "Bumping IT because it is at the front that we are deleting");
-                ++(*iterators_it);
-              }
-            }  // end foreach iterator
-
-            pktQueue.pop_front();
-            packet_counts[zm_packet->packet.stream_index] -= 1;
-            Debug(1,
-                  "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
-                  zm_packet->packet.stream_index,
-                  zm_packet->image_index,
-                  zm_packet->keyframe,
-                  packet_counts[video_stream_id],
-                  max_video_packet_count,
-                  pktQueue.size());
-          } // end while
-        }
-      }  // end if too many video packets
-      if (max_video_packet_count > 0) {
-        while (packet_counts[video_stream_id] > max_video_packet_count) {
-          Error("Unable to free up older packets.  Waiting.");
-          condition.notify_all();
-          condition.wait(lck);
-          if (deleting or zm_terminate)
-            return false;
-        }
-      }
-    }  // end if this packet is a video packet
-
     pktQueue.push_back(add_packet);
     packet_counts[add_packet->packet.stream_index] += 1;
     Debug(2, "packet counts for %d is %d",
@@ -149,25 +94,88 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
         packet_counts[add_packet->packet.stream_index]);
 
     for (
-        std::list<packetqueue_iterator *>::iterator iterators_it = iterators.begin();
+        auto iterators_it = iterators.begin();
         iterators_it != iterators.end();
         ++iterators_it
         ) {
       packetqueue_iterator *iterator_it = *iterators_it;
       if (*iterator_it == pktQueue.end()) {
-        Debug(4, "pointing it %p to back", iterator_it);
         --(*iterator_it);
-      } else {
-        Debug(4, "it %p not at end", iterator_it);
       }
     }  // end foreach iterator
+
+    if (
+        (add_packet->packet.stream_index == video_stream_id)
+        and
+        (max_video_packet_count > 0)
+        and
+        (packet_counts[video_stream_id] > max_video_packet_count)
+       ) {
+      if (!warned_count) {
+        warned_count++;
+        Warning("You have set the max video packets in the queue to %u."
+            " The queue is full. Either Analysis is not keeping up or"
+            " your camera's keyframe interval is larger than this setting."
+            , max_video_packet_count);
+      }
+
+      for (
+          // Start at second packet because the first is always a keyframe
+          auto it = ++pktQueue.begin();
+          //it != pktQueue.end() and  // can't git end because we added our packet
+          (*it != add_packet) && !(deleting or zm_terminate);
+          // iterator is incremented by erase
+      ) {
+        std::shared_ptr <ZMPacket>zm_packet = *it;
+
+        ZMLockedPacket lp(zm_packet);
+        if (!lp.trylock()) {
+          if (warned_count <2) {
+            warned_count++;
+            // Can't delete a locked packet, but can delete one after it.
+            Warning("Found locked packet when trying to free up video packets. This basically means that decoding is not keeping up.");
+          }
+          ++it;
+          continue;
+        }
+
+        for (
+            auto iterators_it = iterators.begin();
+            iterators_it != iterators.end();
+            ++iterators_it
+            ) {
+          auto iterator_it = *iterators_it;
+          // Have to check each iterator and make sure it doesn't point to the packet we are about to delete
+          if (*(*iterator_it) == zm_packet) {
+            Debug(1, "Bumping IT because it is at the front that we are deleting");
+            ++(*iterator_it);
+          }
+        }  // end foreach iterator
+
+        it = pktQueue.erase(it);
+        packet_counts[zm_packet->packet.stream_index] -= 1;
+        Debug(1,
+            "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
+            zm_packet->packet.stream_index,
+            zm_packet->image_index,
+            zm_packet->keyframe,
+            packet_counts[video_stream_id],
+            max_video_packet_count,
+            pktQueue.size());
+
+        if (zm_packet->packet.stream_index == video_stream_id)
+          break;
+      }  // end while
+    } else {
+      warned_count--;
+    }  // end if not able catch up
   }  // end lock scope
   // We signal on every packet because someday we may analyze sound
   Debug(4, "packetqueue queuepacket, unlocked signalling");
   condition.notify_all();
 
 	return true;
-} // end bool PacketQueue::queuePacket(ZMPacket* zm_packet)
+}  // end bool PacketQueue::queuePacket(ZMPacket* zm_packet)
 
 void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
   // Only do queueCleaning if we are adding a video keyframe, so that we guarantee that there is one.
@@ -179,8 +187,9 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
   //
   // So start at the beginning, counting video packets until the next keyframe.  
   // Then if deleting those packets doesn't break 1 and 2, then go ahead and delete them.
+  //
+  // One assumption that we can make is that there will be packets in the queue. Because we call it while holding a locked packet
   if (deleting) return;
-  if (!pktQueue.size()) return;
 
   if (keep_keyframes and ! (
         add_packet->packet.stream_index == video_stream_id
@@ -218,7 +227,10 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
     while ((*pktQueue.begin() != add_packet) and (packet_counts[video_stream_id] > pre_event_video_packet_count + tail_count)) {
       std::shared_ptr<ZMPacket> zm_packet = *pktQueue.begin();
       ZMLockedPacket *lp = new ZMLockedPacket(zm_packet);
-      if (!lp->trylock()) break;
+      if (!lp->trylock()) {
+        delete lp;
+        break;
+      }
       delete lp;
 
       if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
@@ -236,13 +248,12 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
             packet_counts[video_stream_id],
             pre_event_video_packet_count,
             pktQueue.size());
-      //delete zm_packet;
     } // end while
     return;
   }
 
-  packetqueue_iterator it = pktQueue.begin();
-  packetqueue_iterator next_front = pktQueue.begin();
+  auto it = pktQueue.begin();
+  auto next_front = pktQueue.begin();
 
   // First packet is special because we know it is a video keyframe and only need to check for lock
   std::shared_ptr<ZMPacket> zm_packet = *it;
@@ -250,50 +261,53 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
     return;
   }
 
-  Debug(1, "trying lock on first packet");
   ZMLockedPacket *lp = new ZMLockedPacket(zm_packet);
   if (lp->trylock()) {
+    int keyframe_interval = 1;
     int video_packets_to_delete = 0;    // This is a count of how many packets we will delete so we know when to stop looking
-    Debug(1, "Have lock on first packet");
+    Debug(4, "Have lock on first packet");
     ++it;
     delete lp;
 
-    if (it == pktQueue.end()) {
-      Debug(1, "Hit end already");
-      it = pktQueue.begin();
-    } else {
     // Since we have many packets in the queue, we should NOT be pointing at end so don't need to test for that
     while (*it != add_packet) {
       zm_packet = *it;
       lp = new ZMLockedPacket(zm_packet);
       if (!lp->trylock()) {
+        Debug(3, "Failed locking packet %d", zm_packet->image_index);
         delete lp;
         break;
       }
       delete lp;
 
-      if (is_there_an_iterator_pointing_to_packet(zm_packet) and (pktQueue.begin() == next_front)) {
-        Warning("Found iterator at beginning of queue. Some thread isn't keeping up");
+#if 0
+      // There are no threads that follow analysis thread.  So there cannot be an it pointing here
+      if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
+        if (pktQueue.begin() == next_front)
+          Warning("Found iterator at beginning of queue. Some thread isn't keeping up");
         break;
       }
+#endif
 
       if (zm_packet->packet.stream_index == video_stream_id) {
         if (zm_packet->keyframe) {
-          Debug(3, "Have a video keyframe so setting next front to it");
+          Debug(1, "Have a video keyframe so setting next front to it. Keyframe interval so far is %d", keyframe_interval);
+          keyframe_interval = 1;
           next_front = it;
+        } else {
+          keyframe_interval++;
         }
         ++video_packets_to_delete;
-          Debug(4, "Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
-              video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
+        Debug(3, "Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
+            video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
         if (packet_counts[video_stream_id] - video_packets_to_delete <= pre_event_video_packet_count + tail_count) {
           break;
         }
       }
       ++it;
     } // end while
-    }
   }  // end if first packet not locked
-  Debug(1, "Resulting pointing at latest packet? %d, next front points to begin? %d",
+  Debug(1, "Resulting it pointing at latest packet? %d, next front points to begin? %d",
       ( *it == add_packet ),
       ( next_front == pktQueue.begin() )
       );
@@ -315,7 +329,6 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
             pktQueue.size());
       pktQueue.pop_front();
       packet_counts[zm_packet->packet.stream_index] -= 1;
-      //delete zm_packet;
     }
   }  // end if have at least max_video_packet_count video packets remaining
   // We signal on every packet because someday we may analyze sound
@@ -323,6 +336,11 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
 	return;
 } // end voidPacketQueue::clearPackets(ZMPacket* zm_packet)
 
+void PacketQueue::stop() {
+  deleting = true;
+  condition.notify_all();
+}
+
 void PacketQueue::clear() {
   deleting = true;
   condition.notify_all();
@@ -336,17 +354,17 @@ void PacketQueue::clear() {
     // Someone might have this packet, but not for very long and since we have locked the queue they won't be able to get another one
     ZMLockedPacket *lp = new ZMLockedPacket(packet);
     lp->lock();
-      Debug(1,
-            "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
-            packet->packet.stream_index,
-            packet->image_index,
-            packet->keyframe,
-            packet_counts[video_stream_id],
-            pre_event_video_packet_count,
-            pktQueue.size());
+    Debug(1,
+        "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
+        packet->packet.stream_index,
+        packet->image_index,
+        packet->keyframe,
+        packet_counts[video_stream_id],
+        pre_event_video_packet_count,
+        pktQueue.size());
+    packet_counts[packet->packet.stream_index] -= 1;
     pktQueue.pop_front();
     delete lp;
-    //delete packet;
   }
   Debug(1, "Packetqueue is clear, deleting iterators");
 
